@@ -2,7 +2,7 @@
 #
 #   strip.sh - Strip debugging symbols from binary files
 #
-#   Copyright (c) 2007-2016 Pacman Development Team <pacman-dev@archlinux.org>
+#   Copyright (c) 2007-2018 Pacman Development Team <pacman-dev@archlinux.org>
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -32,7 +32,12 @@ tidy_modify+=('tidy_strip')
 
 
 build_id() {
-	LANG=C readelf -n $1 | sed -n '/Build ID/ { s/.*: //p; q; }'
+	LANG=C readelf -n "$1" | sed -n '/Build ID/ { s/.*: //p; q; }'
+}
+
+source_files() {
+	LANG=C readelf "$1" --debug-dump | \
+		awk '/DW_AT_name +:/{name=$8}/DW_AT_comp_dir +:/{print $8 "/" name}'
 }
 
 strip_file() {
@@ -50,6 +55,15 @@ strip_file() {
 			return
 		fi
 
+		# copy source files to debug directory
+		local f t
+		while read -r t; do
+			f=${t/${dbgsrcdir}/"$srcdir"}
+			mkdir -p "${dbgsrc/"$dbgsrcdir"/}${t%/*}"
+			cp -- "$f" "${dbgsrc/"$dbgsrcdir"/}$t"
+		done < <(source_files "$binary")
+
+		# copy debug symbols to debug directory
 		mkdir -p "$dbgdir/${binary%/*}"
 		objcopy --only-keep-debug "$binary" "$dbgdir/$binary.debug"
 		objcopy --add-gnu-debuglink="$dbgdir/${binary#/}.debug" "$binary"
@@ -88,8 +102,11 @@ tidy_strip() {
 		[[ -z ${STRIP_STATIC+x} ]] && STRIP_STATIC="-S"
 
 		if check_option "debug" "y"; then
-			dbgdir="$pkgdir-debug/usr/lib/debug"
-			mkdir -p "$dbgdir"
+
+			dbgdir="$pkgdirbase/$pkgbase-debug/usr/lib/debug"
+			dbgsrcdir="${DBGSRCDIR:-/usr/src/debug}"
+			dbgsrc="$pkgdirbase/$pkgbase-debug$dbgsrcdir"
+			mkdir -p "$dbgdir" "$dbgsrc"
 		fi
 
 		local binary strip_flags
